import { DesignSystem } from "../design-system";
import { ComponentStyles } from "@microsoft/fast-jss-manager";
import {
    applyFocusVisible,
    directionSwitch,
    format,
    subtract,
    toPx,
} from "@microsoft/fast-jss-utilities";
import { height, heightNumber, horizontalSpacing } from "../utilities/density";
import {
    accentFillRest,
    neutralFocus,
    neutralForegroundActive,
    neutralForegroundHover,
    neutralForegroundRest,
} from "../utilities/color";
import { PivotClassNameContract } from "@microsoft/fast-components-class-name-contracts-msft";
import { applyCornerRadius, applyFocusPlaceholderBorder } from "../utilities/border";
import { applyScaledTypeRamp } from "../utilities/typography";
import { focusOutlineWidth } from "../utilities/design-system";
import { applyCursorPointer } from "../utilities/cursor";

const activeIndicatorHeight: number = 3;
const styles: ComponentStyles<PivotClassNameContract, DesignSystem> = {
    pivot: {
        position: "relative",
        overflow: "hidden",
        color: neutralForegroundRest,
        transition: "all 0.2s ease-in-out",
        display: "flex",
        flexDirection: "column"
    },
    pivot_tabList: {
        display: "flex",
        boxSizing: "border-box",
        position: "relative",
        transition: "all 0.2s ease-in-out",
        "&::before": {
            content: "''",
            height: toPx(activeIndicatorHeight),
            background: "red",
            position: "absolute",
            transition: "all 0.2s ease-in-out",
            bottom: "3px",
            width: "20px",
            ...applyCornerRadius(),
        }
    },
    pivot_tab: {
        ...applyCursorPointer(),
        height: height(),
        padding: format("0 {0}", horizontalSpacing(focusOutlineWidth)),
        whiteSpace: "nowrap",
        display: "flex",
        ...applyFocusPlaceholderBorder(),
        alignItems: "center",
        boxSizing: "border-box",
        userSelect: "none",
        color: neutralForegroundRest,
        ...applyCornerRadius(),
        "&:hover": {
            color: neutralForegroundHover,
        },
        "&:active": {
            color: neutralForegroundActive,
        },
        ...applyFocusVisible<DesignSystem>({
            borderColor: neutralFocus,
        }),
    },
    pivot_tab__active: {},
    pivot_tabContent: {
        transition: "all 0.2s ease-in-out",
        ...applyScaledTypeRamp("t7"),
        position: "relative",
        top: "-2px",
    },
    pivot_activeIndicator: {
        position: "absolute",
        ...applyCornerRadius(),
        top: toPx(subtract(heightNumber(1), activeIndicatorHeight, focusOutlineWidth)),
        left: "-10px",
        transition: "0.2s ease-in-out",
        width: "20px",
        height: toPx(activeIndicatorHeight),
        display: "block",
        background: accentFillRest,
    },
    pivot_tabPanel: {
        display: "block",
    },
    pivot_tabPanel__hidden: {
        display: "none",
    },
    pivot_tabPanels: {
        animationTimingFunction: "cubic-bezier(0.4, 0.0, 0.6, 1.0)",
    },
    pivot_tabPanels__animatePrevious: {
        animation: format("{0} 0.2s", directionSwitch("fromLeft", "fromRight")),
    },
    pivot_tabPanels__animateNext: {
        animation: format("{0} 0.2s", directionSwitch("fromRight", "fromLeft")),
    },
    pivot_tabPanelContent: {},
    "@keyframes fromRight": {
        "0%": {
            opacity: "0",
            transform: "translateX(-50px)",
        },
        "100%": {
            opacity: "1",
            transform: "translateX(0)",
        },
    },
    "@keyframes fromLeft": {
        "0%": {
            opacity: "0",
            transform: "translateX(50px)",
        },
        "100%": {
            transform: "translateX(0)",
            opacity: "1",
        },
    },
};

export default styles;
